@@ -5,9 +5,10 @@ use warnings;
 use base 'Module::Build';
 
 use lib "inc";
-use File::Spec::Functions qw(catfile);
+use File::Spec::Functions qw(catfile rel2abs);
 use ExtUtils::Command;
 use File::Fetch;
+use File::Find;
 use File::Temp qw(tempdir tempfile);
 use Digest::SHA qw(sha1_hex);
 use Archive::Extract;
@@ -283,13 +284,29 @@ sub apply_patch {
 sub do_system_output_tail {
   my ($self, $limit, @cmd) = @_;
   my $output;
-  print "CMD: " . join(' ',@cmd) . "\n";
-  print "Running (stdout+stderr redirected)...\n";
-  my $rv = run3 \@cmd, \undef, \$output, \$output;
+  print STDERR "CMD: " . join(' ',@cmd) . "\n";
+  print STDERR "Running (stdout+stderr redirected)...\n";
+  my $rv = run3(\@cmd, \undef, \$output, \$output, { return_if_system_error => 1 } );
+  my $success = ($rv == 1 && $? == 0) ? 1 : 0;
   $output = substr $output, -$limit if defined $limit; # we want just last N chars
-  print ( (defined $limit) ? "OUTPUT: (only last $limit chars)\n" : "OUTPUT:\n");
-  print $output, "\n";
-  return $rv;
+  if (!defined($limit)) {
+    print STDERR "OUTPUT:\n", $output, "\n";
+  }
+  elsif ($limit>0) {
+    print STDERR "OUTPUT: (only last $limit chars)\n", $output, "\n";
+  }
+  return $success;
+}
+
+sub find_file {
+  my ($self, $dir, $re) = @_;
+  my @files;
+  $re ||= qr/.*/;
+  {    
+    #no warnings 'File::Find'; #hide warning "Can't opendir(...): Permission denied
+    find({ wanted => sub { push @files, rel2abs($_) if /$re/ }, follow => 1, no_chdir => 1 , follow_skip => 2}, $dir);
+  };
+  return @files;
 }
 
 1;
